/*

pad.h - defines external interface to pad processor subsystem.

This set of function calls and data structures provides the top-level 
abstaction for the PAD processor functions. 

The access functions prototyped here are delivered in the libpad.a library.

*/
#ifndef _HAVE_PAD_H
#define _HAVE_PAD_H 1

#include "ie_types.h"

// Added these constants - their location (to another #include file) may change later
static 
const INT16U X_ORIGIN = 151;
static
const INT16U max_pen_x = 875;
static 
const INT16U Y_ORIGIN = 160;
static 
const INT16U max_pen_y = 830;
static 
const INT16U Z_ORIGIN = 300;

// SetConfig and GetStat share the following structure

typedef struct {    
    INT32U filterMode;        
    INT32U touchMode;
    INT32U sampleRate;        // Range 1-4
    INT32U oversamples;       // Range 3-15 or 1
    INT32U delayAD;           // Calibration constant
    INT32U zThreshold;        // Calibration constant
} ito_param_t;


////////////////////////////////////////////////////////////////////////
// int PadInit(void);
//
//  Initialize the Pad subsystem. All data structures are created.
//  Communication with the pad processor is validated.
//
//    Returns 0 on success or -1 on error. pad_errno set on error.
//
//   Possible errors:
//       EPADINIT  
//       EPADEXCEP 
//       EPADCOMM  
//
int PadInit(void);

////////////////////////////////////////////////////////////////////////
// int PadRun(void);
//
//  Start the Pad processor. This function puts the Pad subsystem into a known
//  running state, ready to deliver data or status to the calling program, or to
//  accept a configuration from the caller.
//
//  Returns 0 on success or -1 on error. pad_errno set on error.
//
//   Possible errors:
//       EPADINIT  
//       EPADEXCEP 
//       EPADSTATE 
//
int PadRun(void);


////////////////////////////////////////////////////////////////////////
// int PadReset(void);
//
// Reset the Pad Processor from an error state. Clears all outstanding errors and
// re-initializes the Pad Subsystem. Resets all configuration settings to defaults.
//
// Calling PadReset is the only way to clear most raised exceptions. After clearing 
// exception, Pad Processor must be re-started with a PadRun() command and optionally
// re-configured with a PadSetConfig() command.                                      
//
//  Returns 0 on success or -1 on error. pad_errno set on error.
//
//   Possible errors:
//       EPADINIT  
//       EPADEXCEP - Exception Raised on re-initialization. This generally means
//                   that an unrecoverable condition has arisen.
//
int PadReset(void);


////////////////////////////////////////////////////////////////////////
// int PadStop(void);
//
// Stop the Pad Processor from sending more data. Places the processor in suspend mode.
// Pad can be re-started with a PadRun() command and will retain the configuration 
// settings last sent.
//
//  Returns 0 on success or -1 on error. pad_errno set on error.
//
//   Possible errors:
//       EPADINIT  
//       EPADEXCEP 
//       EPADSTATE 
//
int PadStop(void);


////////////////////////////////////////////////////////////////////////
// int PadSetConfig( pad_config_t *);
//
// Sends configuration settings to Pad Processor.
//
// Configuration settings may be sent while Pad is running or suspended. But for some
// settings it is impossible to know which data points were generated with which 
// configuration. If PadSetConfig is called after an Init or Reset, but before PadRun
// has been called, the Pad processor will deliver data only after setting all of the 
// specified configuration parameters. 
//
// Returns 0 on success or -1 on error. pad_errno set on error return.
//
//   Possible errors:
//       EPADINIT  
//       EPADEXCEP 
//       EPADPTR  
//

typedef struct {
    INT32U      configMask;        // Which fields to process?
    INT32U      readTimeout;        // How long in seconds to wait for data
                                   // After turning on the pad with a GO signal
    ito_param_t ito;
} pad_config_t;

int PadSetConfig( pad_config_t *);

// Configuration Mask Bits 
//(also used in exceptionDetails to detail a range check exception)
#define _CONFIG_MASK        0x00000001  // Unused in SetConfig(). This bit is set in the 
                                        // exceptionDetails error, whenever the
                                        // Caller passes an illegal mask to a SetConfig()
#define _CONFIG_FILTER      0x00000002
#define _CONFIG_TOUCH       0x00000004
#define _CONFIG_ADSETTLE    0x00000008
#define _CONFIG_SRATE       0x00000010
#define _CONFIG_OSAMP       0x00000020
#define _CONFIG_ZTHRESH     0x00000040
#define _CONFIG_TIMEOUT     0x00000080

// Range Check Values

#define PAD_ZTHRESH_MIN 1
#define PAD_ZTHRESH_MAX 1022

#define PAD_SAMPLE_MIN 1
#define PAD_SAMPLE_MAX 4

#define PAD_OSAMP_MIN 1
#define PAD_OSAMP_MAX 15

#define PAD_DELAY_MIN 1
#define PAD_DELAY_MAX 63

// Seconds - timeout on the pad read() function
#define PAD_TIMEOUT_MIN 1
#define PAD_TIMEOUT_MAX 60



////////////////////////////////////////////////////////////////////////
// int PadGetData( ito_event_t * data, int count, int flags);
//
// Get one or more data points from the Pad processor.
//
//  Returns 0 if no events read.
//  Returns a positive integer <= count equal to number of points read.
//  Returns -1 on error or receipt of any signal. 
//    If flag is set to PAD_NOWAIT and no data is present, function will return 0.
//    If flag is set to 0, read will wait for a point. If a point is read, read will
//    return 1 and set data structure to point to event. If, during wait, the calling 
//    process receives a signal, read will return -1. system errno will be EINTR in this
//    case, and pad_errno will be 0.
//
//   If the calling process sends a status message, PadGetData will return 0.
//   The contents of that status message will be returned by a call to PadGetStatus();
//
//  If Pad subsystem raises an exception, PadGetData will return -1 before attempting to
//  read any data. pad_errno will be set to EPADEXCEPTION, and the details of the exception
//  can be obtained by calling PadStat().
//
//   Possible errors:
//       EPADINIT  
//       EPADEXCEP 
//       EPADSTATE 
//       EPADPTR  
//


typedef struct {

    INT32U timestamp;       // 1/1200ths of a second indexed from PIC start.
    INT32U penDown;         // 0 here is a PenUp message except in streaming mode
    INT32U penX;            // 12 LSBs valid
    INT32U penY;            // 12 LSBs valid
    INT32U penZ;            // 12 LSBs valid
} pad_event_t;

#define PAD_GET_NOWAIT  1
#define PAD_PENDATA_MASK 0x00000fff

int PadGetData( pad_event_t*, int);

////////////////////////////////////////////////////////////////////////
// int PadGetStat( pad_stat_t *);
//
// Collects status of the Pad subsystem. Status includes current opperating parameters,
// current operating state, and any exception conditions.
//
// Returns 0 on success or -1 on error. pad_errno set on error return. 
//
//   Possible errors:
//       EPADINIT  
//       EPADSTATE   
//       EPADPTR  
//

//
// Information from the ITO that the Master Prog cannot modify directly.
//

typedef struct {
    char fwVersion[8];    // Version of Firmware running on the PIc processor
    char swVersion[8];    // Version of Pad Processor software running
    char itoVersion[8];   // Version of ito driver 
    INT32U tics2secs;     // Divisor - how many clock ticks make up a second
    INT32U timestamp;     // time ticks since device open();
    INT32U unreadPoints;  // How many unread PenData points are in the buffer.
} ito_info_t;

typedef struct {
    INT32U exceptionBits;    // nonzero means error condition. 
    INT32U exceptionDetail; // Details of raised exceptions
    INT32U itoRunning;      // Set if the PAD is currently collecting Samples
    INT32U readTimeout;     // Timeout on read() of the ITO device
    ito_info_t  info;       // Non-configurable information
    ito_param_t param;      // State of configurable parameters
} pad_stat_t;

int PadGetStat( pad_stat_t *);


////////////////////////////////////////////////////////////////////////
//
// Use of pad_errno vs. raising exceptions.
//
// pad_errno is primarily used to report programmatic errors in use of the Pad Processor
// access functions. (calling functions in in the wrong sequence, passing uninitialized
// pointers, etc. These errors are generally unrecoverable and are the result of a
// programming mistake on the part of the firmware developer.
//
// If the access library cannot establish a link to the Pad Processor, pad_errno is set.
//
// All other systemic errors (including range errors, buffer overflows, HW failures, etc.)
// are encapsulated in exceptions and passed up the chain. The master state machine can
// decide to pass the error on to the host, or to reset the pad processor.
//
// Clearing Exceptions
//
// Exceptions are cleared when the pad processor is reset. Range errors are a special
// case. Data out of range errors are reported as exceptions. These parameters come
// from the host and as such are out of firmware's hands. If an attempt is made to set
// an operating parameter to an illegal value via PadSetConfig(), the command will fail
// and a data-out-of range exception will be raised. This exception will remain in effect
// and no other operations will be permitted until a PadSetConfig() call is made which
// explicitly sets the failed parameter to a valid value.
//
// Attempts to set parameters to illegal values are the only kinds of exceptions which
// are recoverable without commanding a pad subsystem reset.  Invoking a PadReset will
// clear all exceptions including range exceptions, and will reset all operating 
// parameters to power-on default values.
//
// If an exception was raised because of a hardware failure, and that failure is still
// present after a reset, the exception will be raised again. If the hardware failure
// does not resurface after a reset (brownout, etc.), the reset will cure the problem.
//
//

// pad_errno definitions
#define    EPADINIT  1 // Pad Processor not initialized (or PadInit() called repeatedly)
#define    EPADCOMM  2 // Cannot establish communication with Pad Processor
#define    EPADSTATE 3 // Improper function call for current Pad Processor state.
#define    EPADPTR   4 // Null pointer passed as parameter
#define    EPADEXCEP 5 // Exception Condition exists. Exception must be cleared.

extern int  pad_errno;

//! PadPrintException( int bits, int details );
//
//  Convenience function to print out meanings of exception bits
void
PadPrintException( int bits, int details);

// Exception Definitions

// Primary Exception Indicator

#define PADEX_ITO_FIFO_OF  0x00000001  // ITO Driver: SPI hardware fifo overflow
#define PADEX_ITO_DATA     0x00000002  // ITO Driver: Bad Packet read from SPI
#define PADEX_ITO_TIMEOUT  0x00000004  // ITO Driver: Timeout waiting for PIC message 
#define PADEX_ITO_SELFTEST 0x00000008  // ITO Driver: uController selftest failed (More)
#define PADEX_BUFFER_OF    0x00000010  // ITO driver: ring buffer overflow
#define PADEX_BADSIG       0x00000020  // Pad Processor: received unexpected signal
#define PADEX_BADMSG       0x00000040  // Pad Processor: received unexpected IPC Message
#define PADEX_PAD_TIMEOUT  0x00000200  // Pad Processor: Timeout on read() of Pen Data
#define PADEX_MSGRANGE     0x00000080  // Pad Library: Bad Configuration Parameter (More)
#define PADEX_SYSERR       0x00000100  // Pad Library: Error on System, Call (More)  

// Secondary Exception Details

// Bits 0-6 are used to report range errors in config messages
#define PADEX_RANGE_MASK    _CONFIG_MASK        
#define PADEX_RANGE_FILTER  _CONFIG_FILTER   
#define PADEX_RANGE_TOUCH   _CONFIG_TOUCH   
#define PADEX_RANGE_SRATE   _CONFIG_SRATE   
#define PADEX_RANGE_OSAMP   _CONFIG_OSAMP      
#define PADEX_RANGE_ADSETL  _CONFIG_ADSETTLE   
#define PADEX_RANGE_ZTHRES  _CONFIG_ZTHRESH 
#define PADEX_RANGE_TIMEOUT _CONFIG_TIMEOUT

// bits 8-15 are used to report system call failures

#define PADEX_SYS_MSGGET    0x00000100
#define PADEX_SYS_MSGSND    0x00000200
#define PADEX_SYS_MSGRCV    0x00000400
#define PADEX_SYS_KILL      0x00000800
#define PADEX_SYS_PID       0x00001000   // Cannot locate the PID of the pad processor
#define PADEX_SYS_RES1      0x00002000   // reserved unused
#define PADEX_SYS_RES2      0x00004000   // reserved unused
#define PADEX_SYS_RES3      0x00008000   // reserved unused


// bits 16-20 are used to report ITO Selftest failures

#define PADEX_STEST_WDT     0x00010000
#define PADEX_STEST_BRN     0x00020000
#define PADEX_STEST_ADC     0x00040000
#define PADEX_STEST_OSC     0x00080000
#define PADEX_STEST_STK     0x00100000

// bits 21-24 are used to report device driver access failures

#define PADEX_DVR_OPEN      0x00200000
#define PADEX_DVR_IOCTL     0x00400000
#define PADEX_DVR_READ      0x00800000
#define PADEX_DVR_FLUSH     0x01000000

// bits 25-29 encapsulate the unexpected signal number received
#define PADEX_BADSIG_MASK   0x3e000000
#define PADEX_BADSIG_SHIFT  25

// bits 30-31 unassigned

#endif // _HAVE_PAD_H

